@@ -51,7 +51,7 @@ import os
 import deprecated as _deprecated
 from . import config
 from . import core
-from .data_structures import Datatable, TableSet, read_csv, read_excel
+from .data_structures import Datatable, TableSet, DataSet, read_csv, read_excel
 
 try:
     from . import database
@@ -69,6 +69,7 @@ from . import interfaces
 from . import util as util
 from . import admin as admin
 from . import templates
+from . import converters
 # from . import utilities as _util 
 
 from . import data_readers
@@ -92,18 +93,20 @@ if db_connected:
     commitTable  = core.DB.commitTable
     commitTables = core.DB.commitTables
     
-    updateTable  = core.DB.updateTable
-    updateTables = core.DB.updateTables
+    updateTable             = core.DB.updateTable
+    updateTables            = core.DB.updateTables
+    updateTablesAvailable   = core.DB.updateTablesAvailable
     
-    removeTable  = core.DB.removeTable
+    removeTable   = core.DB.removeTable
     removeTables  = core.DB.removeTables
     
-    find         = _deprecated.deprecated(core.DB.findc, version='0.4.7', reason="Will be removed, please use finc instead")
-    findc        = core.DB.findc
-    findp        = core.DB.findp
-    findExact    = core.DB.findExact
-    getTable     = core.DB.getTable
-    getTables    = core.DB.getTables
+    find                = _deprecated.deprecated(core.DB.findc, version='0.4.7', reason="Will be removed, please use finc instead")
+    findc               = core.DB.findc
+    findp               = core.DB.findp
+    findExact           = core.DB.findExact
+    getTable            = core.DB.getTable
+    getTables           = core.DB.getTables
+    getTablesAvailable  = core.DB.getTablesAvailable
     
     isAvailable  = core.DB._tableExists
 
