@@ -33,7 +33,7 @@ class Datatable(pd.DataFrame):
     def __init__(self, *args, **kwargs):
         
         # pop meta out of kwargs since pandas is not expecting it
-        overwrite_meta = kwargs.pop('meta', {}) # retrun empty dict if no meta is provide
+        overwrite_meta = kwargs.pop('meta', {}) # return empty dict if no meta is provide
         
         
         
@@ -63,8 +63,8 @@ class Datatable(pd.DataFrame):
         if config.AVAILABLE_XARRAY:
             self.to_xarray = self._to_xarray
             
-        self.columns.name = 'time'
-        self.index.name   = 'region'
+        self.columns.name = config.DATATABLE_COLUMN_NAME
+        self.index.name   = config.DATATABLE_INDEX_NAME
         
         # print(self.meta)
         if ('_timeformat' in self.meta.keys()) and (self.meta['_timeformat'] != '%Y'):
@@ -649,6 +649,8 @@ class Datatable(pd.DataFrame):
         datatable
 
         """
+        kwargs.setdefault("sort", True)
+
         if isinstance(other,Datatable):
             
             if other.meta['entity'] != self.meta['entity']:
@@ -658,7 +660,7 @@ class Datatable(pd.DataFrame):
             if other.meta['unit'] != self.meta['unit']:
                 other = other.convert(self.meta['unit'])
         
-        out =  super(Datatable, self).append(other, **kwargs)
+        out =  pd.concat([self, other], **kwargs)
         
         # only copy required keys
         out.meta = {key: value for key, value in self.meta.items() if key in config.REQUIRED_META_FIELDS}
@@ -866,6 +868,7 @@ class TableSet(dict):
             except:
 #                print('Could not generate ID, key used instead')
                 datatable.ID = key
+        self.inventory.loc[datatable.ID] = None
         self.inventory.loc[datatable.ID, "key"] = key
         self.inventory.loc[datatable.ID, config.INVENTORY_FIELDS] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
     
@@ -1107,7 +1110,7 @@ class TableSet(dict):
         
         
     
-        #%%
+    #%%
         
     def to_compact_excel(self,
                          writer,
@@ -1124,7 +1127,7 @@ class TableSet(dict):
             need_close = True
             
         
-        long, metaDict = self.to_compact_long_format(include_id)
+        long, metaDict = self.to_compact_long_format(include_id, meta_columns=meta_columns)
     
         if meta_columns is not None:
             for metaKey in meta_columns:
@@ -1269,9 +1272,11 @@ class TableSet(dict):
     
     
     def to_compact_long_format(self, 
-                                include_id= False):
+                                include_id= False,
+                                meta_columns=None):
         
-        meta_columns = ['region'] + self.get_all_meta_keys()
+        if meta_columns is None:
+            meta_columns = ['region'] + self.get_all_meta_keys()
         # meta_columns = ['region'] + config.INVENTORY_FIELDS
         if include_id:
             meta_columns.append('ID')
@@ -1390,13 +1395,22 @@ class TableSet(dict):
         return [ self[key] for key in self.keys()]
     
         
-    def to_LongTable(self, meta_list = ['variable', 'region','scenario', 'model', 'unit']):
+    def to_LongTable(self, 
+                     native_regions=False,
+                     meta_list = ['variable', 'region','scenario', 'model', 'unit']):
         tables = []
 
         for variable, df in self.items():
             if df.empty:
                 continue
             inp_dict = dict()    
+            
+            if isinstance(df.index, pd.MultiIndex):
+                if native_regions:
+                    df = df.reset_index('standard_region',drop=True)
+                else:
+                    df = df.reset_index('region',drop=True)
+                
             for metaKey in meta_list:
                 if metaKey == 'region':
                     inp_dict[metaKey] = df.index
@@ -1424,6 +1438,13 @@ class TableSet(dict):
         import pyam
         long_table = self.to_LongTable()
         long_table.index.name = None
+        
+        # make sure that region does not contain any nan values
+        na_mask = long_table['region'].isnull()
+        if config.DEBUG and (sum(na_mask) >0 ):
+            print(f'Removing {sum(na_mask)} nan items in region index')
+        long_table = long_table[~na_mask]
+        
         idf = pyam.IamDataFrame(pd.DataFrame(long_table))
 
         meta = pd.DataFrame([df.meta for df in self.values()])
@@ -1696,7 +1717,24 @@ def _try_number_format(x):
         except:
             return x
             
-def read_csv(fileName, load_raw=False):
+def read_csv(fileName, native_regions=False):
+    """
+    Load DataTable from csv file.
+
+    Parameters
+    ----------
+    fileName : str
+        Path of file.
+    native_regions : bool, optional
+            Load native region defintions if available. The default is False.
+
+
+    Returns
+    -------
+    DataTable
+        DataTable with data and meta.
+
+    """
     
     fid = open(fileName,'r', encoding='utf-8')
     
@@ -1709,15 +1747,18 @@ def read_csv(fileName, load_raw=False):
         line = fid.readline()
         if line == config.DATA_DECLARATION:
             break
-        dataTuple = line.replace('\n','').split(',')
-        meta[dataTuple[0]] = _try_number_format(dataTuple[1].strip())
+        try:
+            key, val = line.replace('\n','').split(',', maxsplit=1)
+        except:
+            continue
+        meta[key] = _try_number_format(val.strip())
         if "unit" not in meta.keys():
             meta["unit"] = ""
     
     if 'timeformat' in meta.keys():
          meta['_timeformat'] = meta['timeformat']
          del meta['timeformat']
-    #print(meta) 
+    # print(meta) 
     
     df = pd.read_csv(fid)
     if 'standard_region' not in df.columns:
@@ -1725,26 +1766,29 @@ def read_csv(fileName, load_raw=False):
         df = df.set_index(df.columns[0])
     else:
         # new datatable
-        if load_raw :
+        if native_regions:
             
-            df = df.set_index('region')
-            if 'standard_region' in df.columns:
-                df = df.drop('standard_region',axis=1)
+            df = df.set_index(['region', 'standard_region'])
+            # if :
+                # df = df.drop('standard_region',axis=1)
         else:
-            df = df.set_index('standard_region')
-            df = df.drop('region',axis=1)
+            df = df.set_index(['standard_region', 'region'])
+            df = df.reset_index('region', drop = True)
+            df = df[~df.index.isnull()]
     fid.close()
     df = Datatable(df, meta=meta)
+    
     if ('_timeformat' in meta.keys()) and (meta['_timeformat'] != '%Y'):
         df.columns_to_datetime()
     else:
         df.columns = df.columns.astype(int)
-    dupl_mask = df.index.duplicated()
     
+    
+    dupl_mask = df.index.duplicated()
     #removing duplicated entries that might results in different native and standart region definitions
     if  sum(dupl_mask) > 0:
         if config.DEBUG:
-            print('Removing duplicated entry')
+            print(f'Removing duplicated {sum(dupl_mask)} entry(ies)')
         df = df.iloc[~dupl_mask,:]
     
     return df#.drop_duplicates()
@@ -1873,7 +1917,8 @@ def read_compact_excel(file_name, sheet_name=None):
                 meta = { x: metaDict[x] for x in metaDict.keys() if x not in  ['meta_columns']}
                 
                 for meta_col in meta_columns:
-                    meta[meta_col] = line[meta_col]
+                    if meta_col in line.keys():
+                        meta[meta_col] = line[meta_col]
                 region = line.loc['region']
                 
                 if 'ID' in line.index:
@@ -1927,7 +1972,140 @@ class MetaData(dict):
         super(MetaData, self).__setitem__('pathway', '|'.join([self[key] for key in ['scenario', 'model'] if key in self.keys()]))
         super(MetaData, self).__setitem__('source', '_'.join([self[key] for key in ['institution', 'year'] if key in self.keys()]))
         
+#%% Test     
+def _add_required_meta(data, meta, stacked_dims):
+    
+    for key in stacked_dims.keys():
         
+        dims_to_add = list()
+        for dim in stacked_dims[key]:
+            if dim in data.index.names:
+                continue
+            else:
+                if dim in meta.columns:
+                    dims_to_add.append(dim)
+        data = data.join(meta.loc[:, dims_to_add])
+        idx_names = set(data.index.names).union(dims_to_add)
+        print(dims_to_add)
+        print(idx_names)
+        data = data.reset_index().set_index(list(idx_names))
+        return data
+    
+def _add_meta(data, meta, stacked_dims):
+    
+    data = data.join(meta)
+    # data['meta'] = range(len(data.index))
+    stacked_dims.update({'meta' : list(meta.columns)}) 
+    idx_names = set(data.index.names).union(list(meta.columns))
+    data = data.reset_index().set_index(list(idx_names))
+    return data, stacked_dims            
+
+def _pack_dimensions(index, **stacked_dims):
+    packed_labels = {}
+    packed_values = {}
+    drop_levels = []
+    
+    for dim, levels in stacked_dims.items():
+        labels = pd.MultiIndex.from_arrays([index.get_level_values(l) for l in levels])
+        packed_labels[dim] = labels_u = labels.unique()
+        packed_values[dim] = pd.Index(labels_u.get_indexer(labels), name=dim)
+        drop_levels.extend(levels)
+
+    return (
+        pd.MultiIndex.from_arrays(
+            [index.get_level_values(l) for l in index.names.difference(drop_levels)] +
+            list(packed_values.values())
+        ),
+        packed_labels
+    )
+
+# def _pack_dimensions(index, 
+#                      meta = None, 
+#                      **stacked_dims):
+#     packed_labels = {}
+#     packed_values = {}
+#     drop_levels = []
+    
+#     for dim, levels in stacked_dims.items():
+#         labels = pd.MultiIndex.from_arrays([index.get_level_values(l) for l in levels])
+#         packed_labels[dim] = labels_u = labels.unique()
+#         packed_values[dim] = pd.Index(labels_u.get_indexer(labels), name=dim)
+#         drop_levels.extend(levels)
+
+#     #meta 
+#     if meta is not None:
+#         for st_dim in stacked_dims.keys():
+#             if st_dim in meta.columns:
+#                 meta = meta.copy().drop(st_dim,axis= 1)
+#         labels = pd.MultiIndex.from_arrays([index.get_level_values(l) for l in meta.index.names])
+#         labels_u = labels.unique()
+#         packed_values['meta'] = pd.Index(labels_u.get_indexer(labels), name='meta')
+#         packed_labels['meta'] = pd.MultiIndex.from_arrays(meta.loc[labels_u,:].values.T, names = meta.columns)
+    
+    
+#     return (
+#         pd.MultiIndex.from_arrays(
+#             [index.get_level_values(l) for l in index.names.difference(drop_levels)] +
+#             list(packed_values.values())
+#         ),
+#         packed_labels
+#     )
+
+import xarray as xr
+class DataSet(xr.Dataset):
+    """
+    Very simple class to allow initialization of xarray datasets from pyam, wide pandas dataframes 
+    and datatoolbox queries.
+    """
+    
+    @classmethod
+    def from_wide_dataframe(cls, 
+                            data,
+                            meta = None,
+                            stacked_dims = {'pathway' : ("model", "scenario")}):
+        to_merge = list()
+        for (variable, unit), df_ in data.groupby(['variable', 'unit']):
+            #print(df_)
+            
+            index = df_.index
+            df_ = df_.reset_index(['variable', 'unit'], drop = True)
+            index, dims = _pack_dimensions(df_.index, **stacked_dims)
+            da = (xr.DataArray(df_.set_axis(index)
+                  .rename_axis(columns="year"))
+                  .unstack("dim_0")
+                  .pint.quantify(unit)
+                  .assign_coords(dims)
+                  )
+            to_merge.append(xr.Dataset({variable : da}))
+        self = xr.merge(to_merge)
+        return self
+      
+    @classmethod
+    def from_pyam(cls, 
+                  data,
+                  meta_dims =None,
+                  stacked_dims = {'pathway' : ("model", "scenario")}):
+        
+        if meta_dims is not None:
+            meta = data.meta.loc[:,meta_dims]
+            data, stacked_dims = _add_meta(data, meta, stacked_dims)
+        else:
+            meta = None
+        data = data.timeseries()
+        #data = _add_required_meta(data, meta, stacked_dims)
+        
+        return cls.from_wide_dataframe(data, meta, stacked_dims)
+        
+    @classmethod
+    def from_query(cls,
+                   query,
+                   stacked_dims = {'pathway' : ("model", "scenario")}):
+        dimensions = ['model', 'scenario', 'region', 'variable', 'source', 'unit']
+        data = query.as_wide_dataframe(meta_list = dimensions).set_index(dimensions)
+        
+        return cls.from_wide_dataframe(data, meta = None, stacked_dims = stacked_dims)
+    
+    
 if __name__ == '__main__':
     meta = MetaData()
     meta['entity'] = 'Emissions|CO2'
