@@ -6,13 +6,17 @@ Created on Wed May  8 11:25:15 2019
 @author: Andreas Geiges
 """
 
-import os
+import os, sys
 import networkx as nx
+from functools import reduce
+from operator import and_
 
 import pandas as pd
 import numpy as np
 from typing import Union, Iterable
 
+import logging
+
 import datatoolbox as dt
 from datatoolbox import mapping as mapp
 from datatoolbox import core
@@ -26,6 +30,7 @@ import tqdm
 import deprecated as dp
 #from .tools import kaya_idendentiy_decomposition
 
+logger = logging.getLogger(__name__)
 
 try:
     from hdx.location.country import Country
@@ -168,7 +173,8 @@ def cleanDataTable(dataTable):
     #         ", ".join(invalidSpatialIDs)
     #     )
     #     dataTable = dataTable.loc[dataTable.index.difference(invalidSpatialIDs)]
-    
+    if 'standard_region' in dataTable.columns:
+        dataTable = dataTable.reset_index('region').set_index(['region', 'standard_region'])
     dataTable = dataTable.dropna(how="all", axis=1).dropna(how="all", axis=0).astype(float)
 
     # clean meta data
@@ -1131,8 +1137,29 @@ def plot_query_as_graph(results, savefig_path=None):
 #                 figsize=[5,6],
                  savefig_path=savefig_path)
 
-def to_pyam(results):
-    return core.DB.getTables(results.index).to_pyam()
+def to_pyam(results, native_regions = False, disable_progress=None):
+    """
+    Load resuls as pyam IDateFrame.
+    
+    Parameters
+    ----------
+    results : pandas Dataframe with datatoolbox query results
+        Results from find.
+    native_regions : bool, optional
+        Load native region defintions if available. The default is False.
+    disable_progress : bool, optional
+        Disable displaying of progressbar. The default None hides the
+        progressbar on non-tty outputs.
+
+    Returns
+    -------
+    TYPE
+        DESCRIPTION.
+
+    """
+    return core.DB.getTables(
+        results.index, native_regions, disable_progress=disable_progress
+    ).to_pyam()
 
 def filterp(df, level=None, regex=False, **filters):
         """ 
@@ -1195,6 +1222,21 @@ def yearsColumnsOnly(index):
                 pass
     return newColumns
 
+def isin(df=None, **filters):
+    """Constructs a MultiIndex selector
+
+    Usage
+    -----
+    > df.loc[isin(region="World", gas=["CO2", "N2O"])]
+    or with explicit df to get boolean mask
+    > isin(df, region="World", gas=["CO2", "N2O"])
+    """
+    def tester(df):
+        tests = (df.index.isin(np.atleast_1d(v), level=k) for k, v in filters.items())
+        return reduce(and_, tests, next(tests))
+
+    return tester if df is None else tester(df)
+
 #%%    
 if __name__ == '__main__':
     #%%
@@ -1229,4 +1271,36 @@ if __name__ == '__main__':
 
     outputTables, success = forAll(calculateTotalBiomass, "scenario")
     
-    
\ No newline at end of file
+
+def handle_exception(exc_type, exc_value, exc_traceback):
+    if issubclass(exc_type, KeyboardInterrupt):
+        sys.__excepthook__(exc_type, exc_value, exc_traceback)
+        return
+
+    logger.exception("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))   
+
+def setup_logging(log_uncaught_exceptions=True, **kwargs):
+    from colorlog import ColoredFormatter
+
+    kwargs.setdefault("level", "INFO")
+
+    streamhandler = logging.StreamHandler()
+    streamhandler.setFormatter(
+        ColoredFormatter(
+            "%(name)-12s: %(log_color)s%(levelname)-8s%(reset)s %(message)s",
+            datefmt=None,
+            reset=True,
+        )
+    )
+
+    kwargs.setdefault("handlers", []).append(streamhandler)
+    
+    if log_uncaught_exceptions:
+        sys.excepthook = handle_exception
+
+    logging.basicConfig(**kwargs)
+    
+    def add_parent_to_syspath():
+        
+        if '..' not in sys.path:
+            sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
\ No newline at end of file
